require_relative 'authentication/oauth2'
require_relative 'authentication/token_authentication'

module Asana
  # Public: Authentication strategies for the Asana API.
  module Authentication
  end
end
